@@ -56,8 +56,8 @@ rownames_to_column <- function(df, var = "rowname") {
     stopc("Column `", var, "` already exists")
   }
 
-  new_df <- add_column(df, !!(var) := rownames(df), .before = 1)
-  new_df
+  new_df <- add_column(df, !! var := rownames(df), .before = 1)
+  remove_rownames(new_df)
 }
 
 #' @export
@@ -69,8 +69,8 @@ rowid_to_column <- function(df, var = "rowid") {
     stopc("Column `", var, "` already exists")
   }
 
-  new_df <- add_column(df, !!(var) := seq_len(nrow(df)), .before = 1)
-  new_df
+  new_df <- add_column(df, !! var := seq_len(nrow(df)), .before = 1)
+  remove_rownames(new_df)
 }
 
 #' @rdname rownames
